@@ -2,7 +2,7 @@
 -- server.lua by blastbeat of the luadch project
 -- Re-used here under the MIT/X Consortium License
 -- 
--- Modifications (C) 2008-2009 Matthew Wild, Waqas Hussain
+-- Modifications (C) 2008-2010 Matthew Wild, Waqas Hussain
 --
 
 -- // wrapping luadch stuff // --
@@ -434,7 +434,7 @@ wrapconnection = function( server, listeners, socket, ip, serverport, clientport
     handler.bufferlen = function( readlen, sendlen )
         maxsendlen = sendlen or maxsendlen
         maxreadlen = readlen or maxreadlen
-        return maxreadlen, maxsendlen
+        return bufferlen, maxreadlen, maxsendlen
     end
     handler.lock = function( switch )
         if switch == true then
@@ -467,7 +467,7 @@ wrapconnection = function( server, listeners, socket, ip, serverport, clientport
     end
     local _readbuffer = function( )    -- this function reads data
         local buffer, err, part = receive( socket, pattern )    -- receive buffer with "pattern"
-        if not err or ( err == "timeout" or err == "wantread" ) then    -- received something
+        if not err or (err == "wantread" or err == "timeout") or string_len(part) > 0 then    -- received something
             local buffer = buffer or part or ""
             local len = string_len( buffer )
             if len > maxreadlen then
@@ -698,7 +698,7 @@ addserver = function( listeners, port, addr, pattern, sslctx, maxconnections, st
     if type( listeners ) ~= "table" then
         err = "invalid listener table"
     end
-    if not type( port ) == "number" or not ( port >= 0 and port <= 65535 ) then
+    if type( port ) ~= "number" or not ( port >= 0 and port <= 65535 ) then
         err = "invalid port"
     elseif _server[ port ] then
         err =  "listeners on port '" .. port .. "' already exist"
